#ifndef _HBOOK_HH_
#define _HBOOK_HH_

typedef float FitFloat;

extern "C" {
  void hbook1_( const int& id,
		const char* chtitl,
		const int& nx,
		const float& xmi,
		const float& xma,
		const float& vmx,
		const int lenchtitl );
  
  void hbook2_( const int& id,
		const char* chtitl,
		const int& nx,
		const float& xmi,
		const float& xma,
		const int& ny,
		const float& ymi,
		const float& yma,
		const float& vmx,
		const int lenchtitl );

  void hbooknc_( const int& id,
		 const char* chtitl,
		 const int& nvar,
		 const char* block,
		 const float* tuple,
		 const char* chtags,
		 const int lenchtitl, 
		 const int lenblock,
		 const int lenchtags);

  void hbnt_( const int& id,
	      const char* chtitl,	      
	      const char* chopt,
	      const int lenchtitl,
	      const int lenchtitm);

  void hfnt_( const int& id);

  void hfill_( const int& id,
	       const float& x,
	       const float& y,
	       const float& weight );
		
  void hprint_( const int& id );
		
  void hrout_( const int &id,
	       int & icycle,
	       char * opt,
	       int len );

  void hbslix_( const int& id,
		const int& nslit,
		const float& vmx );

  void hfithn_( int const& id,
		char const * chfun,
		char const * chopt,
		int const& np,
		FitFloat* par,
		FitFloat const * step,
		FitFloat const * pmin,
		FitFloat const * pmax,
		FitFloat * sigpar,
		FitFloat & chi2,
		int chfunlen,
		int choptlen );

  void hfitga_( int const& id,
		float& a,
		float& b,
		float& c,
		float& chi2,
		int const& ic, 
		float* sigpar );

  void hreset_( int const& id,
		char const* chopt,
		int choptlen );

  void hdelet_( const int& id );

  float hrndm1_( const int& id );

  float hi_( const int& id, const int& i );

  float hf1_( const int& id, const float& x, const float& weight );

  float hf2_( const int& id,
	      const float& x,
	      const float& y,
	      const float& weight );

} /* extern "C" */

#include <string.h>

//#include <std/string.h>
#include <vector.h>

inline void hbook1( const int& id,
		    const char* chtitl,
		    const int& nx,
		    const float& xmi,
		    const float& xma,
		    const float& vmx = 0 )
{
  hbook1_( id, chtitl, nx, xmi, xma, vmx,
	   strlen(const_cast<char*>(chtitl)) );
}


inline void hbook2( const int& id,
		    const char* chtitl,
		    const int& nx,
		    const float& xmi,
		    const float& xma,
		    const int& ny,
		    const float& ymi,
		    const float& yma,
		    const float& vmx = 0 )
{
  hbook2_( id, chtitl, nx, xmi, xma, ny, ymi, yma, vmx,
	   strlen( const_cast<char*>(chtitl)) );
}


inline void hbooknc( const int& id,
		     const char* chtitl,
		     const int& nvar,
		     const char* block,
		     const float* tuple,
		     const char* chtags )
{
  hbooknc_( id, chtitl, nvar, block, tuple, chtags,
	    strlen( const_cast<char*>(chtitl)),
	    strlen( const_cast<char*>(block)),
	    strlen( const_cast<char*>(chtags))
	    );
}


inline void hbnt( const int& id,
		  const char* chtitl,
		  const char* chopt )
{
  hbnt_( id, chtitl, chopt,
	 strlen( const_cast<char*>(chtitl)),
	 strlen( const_cast<char*>(chopt)) );
}


inline void hfnt( const int& id)
{
  hfnt_( id );
}


inline void hfill( const int& id,
		   const float& x,
		   const float& y = 0,
		   const float& weight = 1 )
{
  hfill_( id, x, y, weight );
}


inline void hprint( const int& id )
{
  hprint( id );
}


inline void hrout( const int &id,
		   int & icycle,
		   char * opt )
{
  hrout_( id, icycle, opt, strlen(const_cast<char*>(opt)) );
}


inline void hbslix( const int& id,
		    const int& nslit,
		    const float& vmx )
{
  hbslix( id, nslit, vmx );
}


struct FitParameters
{
  FitParameters() {}
  FitParameters( FitFloat p, FitFloat s = 0, FitFloat n = 0, FitFloat x = 0)
    : par( p ), sigpar( 0 ),
    step( s ), pmin( n ), pmax( x ) {}
  
  FitFloat par,
    sigpar,
    step,
    pmin,
    pmax;
};

inline ostream & operator<<( ostream & os, FitParameters const & fi )
{
  return os << '(' << fi.par << ',' << fi.sigpar << ')';
}


inline FitFloat hfithn( const int& id,
		      char const * chfun,
		      char const * chopt,
		      vector<FitParameters> & p
		      )
{
  FitFloat
    chi2,
    par[p.size()],
    step[p.size()],
    pmin[p.size()],
    pmax[p.size()],
    sigpar[p.size()];
  
  for( unsigned int i=0; i<p.size(); i++ ) {
    par[i]    = p[i].par;
    step[i]   = p[i].step;
    pmin[i]   = p[i].pmin;
    pmax[i]   = p[i].pmax;
    sigpar[i] = p[i].sigpar;
  }

  hfithn_( id, chfun, chopt, p.size(), par,
	   step, pmin, pmax, sigpar, chi2,
	   strlen(chfun), strlen(chopt) );
  for( unsigned int i=0; i<p.size(); i++ ) {
    p[i].par    = par[i];
    p[i].sigpar = sigpar[i];
  }
  return chi2;
}


inline float hfitga( int id,
		     float& a,
		     float& b,
		     float& c,
		     float& chi2,
		     int  ic,
		     float* sigpar )
{
  hfitga_( id, a, b, c, chi2, ic, sigpar );
  return chi2;
}


inline void hreset( int id, char const* chopt = " " )
{
  hreset_( id, chopt, strlen( chopt ) );
}


inline void hdelet( int id )
{
  hdelet_( id );
}


inline float hrndm1( int id )
{
  return hrndm1_( id );
}


inline float hi( int id, int i )
{
  return hi_( id, i );
}


inline void hf1( int id, float x, float weight )
{
  hf1_( id, x, weight );
}


inline void hf2( int id, float x, float y, float weight )
{
  hf2_( id, x, y, weight );
}
#endif
