#ifndef __PRIMARYVERTEX_HH
#define __PRIMARYVERTEX_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   31. Aug 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: A class to fit primary vertices by using the SVertex package
//
// changes:
// 31 Aug 2001 (TG) creation
//
// ********************************************************************

#include <iosfwd>
#include <vector>
#include "interfaces/VertexIf.hh"
#include "smatrix/SVertex.hh"

class TrackIf;
class VertexIf;

/** Primary Vertex class.

    @author T. Glebe
*/
//==============================================================================
// PrimaryVertex
//==============================================================================
class PrimaryVertex : public VertexIf {
public:
  /** @name --- Constructors --- */
  ///
  PrimaryVertex();
  ///
  virtual ~PrimaryVertex();

  /** @name --- VertexIf Access methods --- */
  /// vertex $x$ position
  float vx()   const;
  /// vertex $y$ position
  float vy()   const;
  /// vertex $z$ position
  float vz()   const;
  /// vertex $\chi^2$
  float chi2() const;
  /// upper tail $\chi^2$ probability
  float prob() const;
  /// degrees of freedom of vertex fit
  unsigned short int ndf()  const;
  /// no of tracks in vertex
  unsigned short int ntracks() const;
  /// return a track pointer
  const TrackIf* track(unsigned int i) const;
  /// read/write track access
  const TrackIf*& track(unsigned int i);

#ifndef __CINT__
  /// $\vec{x} = (x,y,z)$
  SVector<double,3> vpos() const;
  /// vertex cov. matrix
  const SMatrix<double,3>& VCOV() const;
#endif

  /// $\sigma_x^2$ of vertex
  float vtx_cov_x()   const;
  /// $\sigma_y^2$ of vertex
  float vtx_cov_y()   const;
  /// $\sigma_z^2$ of vertex
  float vtx_cov_z()   const;

  /** @name --- Expert methods --- */
  /// used by operator<<()
  std::ostream& print( std::ostream& ) const;

  /** @name --- Expert functions --- */
  /// do Kalman-Filter fit
  bool findVertexVt();

  vector<TrackIf*> tracks;

private:
  SVertex<20>      vtx;
};

//==============================================================================
// operator <<
//==============================================================================
inline std::ostream& operator<< (std::ostream& os, const PrimaryVertex& vt) {
  return vt.print( os );
}
#endif
