// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   08. Nov 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: include file which handles CLUE ArteTable specializations
//
// changes:
// 08 Nov 2000 (TG) creation
// 25 Nov 2000 (TG) moved getIDX(), getPTR() from arte/arte.hh to here
//
// *****************************************************************************
#ifndef CLUEARTE_HH
#define CLUEARTE_HH

#include "arte/arte.hh"

/** @name --- Global Utility Functions --- */
//==============================================================================
// getIDX:
//==============================================================================
/// calc ArteTable index from ArtePointer
template <typename T>
inline unsigned int getIDX( ArtePointer<T> ptr ) {
  return (ptr - *ArteTable<T>::begin() + 1);
}

//==============================================================================
// getPTR:
//==============================================================================
/// get ArtePointer from ArteTable index
template <typename T>
inline ArtePointer<T> getPTR( unsigned int idx ) {
  return *(ArteTable<T>::begin() + idx - 1);
}

//==============================================================================
// getPTR:
//==============================================================================
///  calc Clue ArtePointer from Arte ArtePointer
template <typename TN, typename TO>
inline ArtePointer<TN> getPTR( const ArtePointer<TO>& ptr ) {
  return *(ArteTable<TN>::begin() + (ptr - *ArteTable<TO>::begin()));
}

/// calc Clue ArtePointer from Arte ArtePointer
template <typename TN, typename TO>
inline ArtePointer<TN> getPTR( const ArteTableIterator<TO>& iter ) {
  return *(ArteTable<TN>::begin() + (iter - ArteTable<TO>::begin()));
}

// right specialization of Clue ArteTables!
#include "clue/ClonesTableSpec.hh"
#endif
