#ifndef TRACK_BASE_HH
#define TRACK_BASE_HH
// **********************************************************************
//
// source:
//
// type:      source code
//
// created:   27. Jan 1997
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: A track interface class
//
// changes:
// 21 Jan 1997 created by I. Siccama, C. Borgmeier
// 03 Nov 2000 (TG) removed #ifdef __CINT__, changed include files
// 20 Nov 2000 (TG) added namespace CLUE
// 12 Jan 2001 (TG) print() return type changed
// 12 Jan 2001 (TG) removed GetLorentzVector()
//
// **********************************************************************

#include <iosfwd>
#include "arte/TObject.hh"

#ifdef NAMESPACE
namespace CLUE {
#endif

  class RecoTrack;

  /** @memo Track interface class
      @author C. Borgmeier
      Base class for PatternMatch, RecoSegment and MC\_Track
  */
  //==============================================================================
  // TrackBase
  //==============================================================================
  class TrackBase: public TObject {
  public:
    ///
    virtual ~TrackBase();
    
    /** @name --- Access methods ---*/
    /// 0 or overloaded
    virtual float mass() const;

    ///
    virtual void lock();
    ///
    virtual void unlock();
    ///
    virtual int locked( int bit = 0 ) const;
    ///
    virtual const RecoTrack* getRecoTrack() const;

    /** @name --- Expert Functions --- */
    ///
    virtual std::ostream& print(std::ostream& os) const;

#ifdef USE_ROOT
    ClassDef(TrackBase,1)
#endif
  }; // end of class TrackBase


  //==============================================================================
  // operator <<
  //==============================================================================
  inline std::ostream& operator<<( std::ostream& os, const TrackBase& t ) {
    return t.print( os );
  }

#ifdef NAMESPACE
} // namespace CLUE
#endif
#endif /* TRACK_BASE_HH */
