
#ifdef USE_ROOT

#include <string.h>
#include <stdio.h>

// *KEEP,TROOT.
#include "TROOT.h"
// *KEEP,TSystem.
#include "TSystem.h"
// *KEEP,TFile.
#include "TFile.h"
// *KEEP,TSelector,T=C++.
#include "TSelector.h"
// *KEEP,TTree.
#include "TTree.h"
// *KEEP,TEventList,T=C++.
#include "TEventList.h"
// *KEEP,TBranchObject,T=C++.
#include "TBranchObject.h"
// *KEEP,TLeafObject,T=C++.
#include "TLeafObject.h"
// *KEEP,TLeaf,T=C++.
#include "TLeaf.h"
// *KEEP,TLeafB,T=C++.
#include "TLeafB.h"
// *KEEP,TLeafC,T=C++.
#include "TLeafC.h"
// *KEEP,TLeafI,T=C++.
#include "TLeafI.h"
// *KEEP,TLeafF,T=C++.
#include "TLeafF.h"
// *KEEP,TLeafS,T=C++.
#include "TLeafS.h"
// *KEEP,TLeafD,T=C++.
#include "TLeafD.h"
// *KEEP,TBasket,T=C++.
#include "TBasket.h"
// *KEEP,TMath.
#include "TMath.h"
// *KEEP,TH2.
#include "TH2.h"
// *KEEP,TF2.
#include "TF2.h"
// *KEEP,TView.
#include "TView.h"
// *KEEP,TPolyMarker.
#include "TPolyMarker.h"
// *KEEP,TPolyMarker3D.
#include "TPolyMarker3D.h"
// *KEEP,TDirectory.
#include "TDirectory.h"
// *KEEP,TBranchClones,T=C++.
#include "TBranchClones.h"
// *KEEP,TClonesArray,T=C++.
#include "TClonesArray.h"
// *KEEP,TClass.
#include "TClass.h"
// *KEEP,TRealData.
#include "TRealData.h"
// *KEEP,TDataMember.
#include "TDataMember.h"
// *KEEP,TDataType.
#include "TDataType.h"
// *KEEP,TVirtualPad.
#include "TVirtualPad.h"
// *KEEP,TProfile.
#include "TProfile.h"
// *KEEP,TMinuit.
#include "TMinuit.h"
// *KEEP,TTreeFormula.
#include "TTreeFormula.h"
// *KEEP,TGaxis.
#include "TGaxis.h"
// *KEEP,TBrowser.
#include "TBrowser.h"
// *KEEP,TStyle.
#include "TStyle.h"
// *KEEP,TChain,T=C++.
#include "TChain.h"
// *KEEP,TProof,T=C++.
#include "TProof.h"
// *KEEP,TProofServ.
#include "TProofServ.h"
// *KEEP,TSocket,T=C++.
#include "TSocket.h"
// *KEEP,TSlave,T=C++.
#include "TSlave.h"
// *KEEP,TMessage,T=C++.
#include "TMessage.h"
// *KEEP,TPacketGenerator,T=C++.
#include "TPacketGenerator.h"
// *KEEP,TInterpreter, T=C++.
#include "TInterpreter.h"
// *KEEP,Foption.
#include "Foption.h"
// *KEND.


static const unsigned int kZombie        = 0x04000000;
extern TTree* gTree;

TBranch* clueBranch( TTree* thisObj,
		     const Text_t *name,
		     const Text_t *classname,
		     void *addobj,
		     Int_t bufsize,
		     Int_t splitlevel)
{
  gTree = thisObj;
// *-*-*-*-*-*-*-*-*-*-*Create a new TTree BranchObject*-*-*-*-*-*-*-*-*-*-*-*
// *-*                  ===============================
//
//    Build a TBranchObject for an object of class classname.
//    addobj is the address of a pointer to an object of class classname.
//    IMPORTANT: classname must derive from TObject.
//
//    This option requires access to the library where the corresponding class
//    is defined. Accessing one single data member in the object implies
//    reading the full object.
//    See the next Branch constructor for a more efficient storage
//    in case the event consists of arrays of identical objects.
//
//    By default the branch buffers are stored in the same file as the Tree.
//    use TBranch::SetFile to specify a different file

   TClass *cl = gROOT->GetClass(classname);
   if (!cl) {
     thisObj->Error("BranchObject","Cannot find class:%s",classname);
     return 0;
   }
   TBranch *branch = new TBranchObject(name,classname,addobj,bufsize,splitlevel);
   thisObj->GetListOfBranches()->Add(branch);
   if (!splitlevel) return branch;

   TObjArray *blist = branch->GetListOfBranches();
   const char *rdname;
   const char *dname;
   char branchname[64];
   if (!cl->GetListOfRealData()) cl->BuildRealData();
   char **apointer = (char**)(addobj);
   TObject *obj = (TObject*)(*apointer);
   Bool_t delobj = kFALSE;
   if (!obj) {
      obj = (TObject*)cl->New();
      obj->SetBit(kZombie);
      delobj = kTRUE;
   }
// *-*- Loop on all public data members of the class and its base classes
   Int_t lenName = strlen(name);
   Int_t isDot = 0;
   if (name[lenName-1] == '.') isDot = 1;
   TBranch *branch1 = 0;
   TRealData *rd;
   TIter      next(cl->GetListOfRealData());
   while ((rd = (TRealData *) next())) {
      TDataMember *dm = rd->GetDataMember();
      if (!dm->IsPersistent()) continue; //do not process members with a ! as the first
                                         // character in the comment field
      rdname = rd->GetName();
      dname  = dm->GetName();

  //  Next line now commented, functionality to process arrays is now implemented
  //  the statement is left to show how to use Property() and kIsArray
  //     if (dm->Property() & kIsArray) continue;

      TDataType *dtype = dm->GetDataType();
      Int_t code = 0;
      if (dtype) code = dm->GetDataType()->GetType();

// *-*- Encode branch name. Use real data member name
      sprintf(branchname,"%s",rdname);
      if (isDot) {
         if (dm->IsaPointer()) sprintf(branchname,"%s%s",name,&rdname[1]);
         else                  sprintf(branchname,"%s%s",name,&rdname[0]);
      }
      char leaflist[64];
      Int_t offset    = rd->GetThisOffset();
      char *pointer   = (char*)obj + offset;
      if (dm->IsaPointer()) {
         TClass *clobj = 0;
         if (!dm->IsBasic()) clobj = gROOT->GetClass(dm->GetTypeName());
         //if (clobj && !strcmp("TClonesArray",clobj->GetName()))
         if (clobj && clobj->InheritsFrom("TClonesArray")) {
            if (isDot) branch1 = new TBranchClones(&branchname[0],pointer,bufsize);
            else       branch1 = new TBranchClones(&branchname[1],pointer,bufsize);
            blist->Add(branch1);
         } else {
            if (!clobj) {
               if (code != 1) continue;
               sprintf(leaflist,"%s/%s",dname,"C");
               branch1 = new TBranch(branchname,pointer,leaflist,bufsize);
               branch1->SetTitle(dname);
               blist->Add(branch1);
            } else {
               if (!clobj->InheritsFrom(TObject::Class())) continue;
	       branch1 = new TBranchObject(dname,clobj->GetName(),pointer,bufsize,splitlevel-1);
               if (isDot) branch1->SetName(&branchname[0]);
               else       branch1->SetName(&branchname[1]);  //do not use the first character (*)
               blist->Add(branch1);
            }
         }
      } else {
// *-*-------------Data Member is a basic data type----------
         if (dm->IsBasic()) {
            if      (code ==  1) sprintf(leaflist,"%s/%s",rdname,"B");
            else if (code == 11) sprintf(leaflist,"%s/%s",rdname,"b");
            else if (code ==  2) sprintf(leaflist,"%s/%s",rdname,"S");
            else if (code == 12) sprintf(leaflist,"%s/%s",rdname,"s");
            else if (code ==  3) sprintf(leaflist,"%s/%s",rdname,"I");
            else if (code == 13) sprintf(leaflist,"%s/%s",rdname,"i");
            else if (code ==  5) sprintf(leaflist,"%s/%s",rdname,"F");
            else if (code ==  8) sprintf(leaflist,"%s/%s",rdname,"D");
            branch1 = new TBranch(branchname,pointer,leaflist,bufsize);
            branch1->SetTitle(rdname);
            blist->Add(branch1);
         }
      }
      if (branch1) branch1->SetOffset(offset);
   }
   if (delobj) delete obj;
   return branch;
}

#endif /* USE_ROOT */
