// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   Mon Jul 16 18:17:47 MEST 2001
//
// author:    glebe
//
// Description: 
//
// changes:
// 14 Sep 2001 (TG) renamed sdecaysTo() to decaysTo()
//
// ********************************************************************

#include "TH1.h"
#include "TH2.h"
#include "TNtuple.h"
#include "TPaveText.h"
#include "BEE.hh"
#include "include/K0Analysis.hh"

int checkPID(const RecoTrack& rtra);

//==============================================================================
// Constructor
//==============================================================================
K0Analysis::K0Analysis(RtEvent* evt) :
 event(evt),
 Piplus(false),
 Piminus(false),
 kshort(false)
{}

//==============================================================================
// EventSelect
//==============================================================================
bool K0Analysis::EventSelect() {
  return true;
}

//==============================================================================
// DefinePattern
//==============================================================================
bool K0Analysis::DefinePattern() {

  Piminus    = pi_minus.createPattern();
  Piplus     = pi_plus.createPattern();

  kshort = K_zero.decaysTo(Piminus & Piplus);
  kshort->with(prob > 0.001 & vz > 0.);

  return true;
}

//==============================================================================
// DefineHistograms
//==============================================================================
bool K0Analysis::DefineHistograms() {

  Kshort = new TNtuple("Kshort","K0s","m");

  return true;
}

//==============================================================================
// Event
//==============================================================================
bool K0Analysis::Event() {

  // clear Particle container
  Piplus->clear();
  Piminus->clear();

  //----------------- select Particles -------------------
  const ArteTableIterator<RecoTrack> RTend = ArteTable<RecoTrack>::end();
  for(ArteTableIterator<RecoTrack> RTit = ArteTable<RecoTrack>::begin();
      RTit != RTend; ++RTit) {
    RecoTrack& rtra = **RTit;

    if(rtra.hsi() < 6 ||
       (rtra.hou() + rtra.hin() < 16)) continue;
    if(rtra.pt()<0.1 || rtra.p() < 2. || fabs(rtra.xf(pi_plus.mass())) > 0.2)
      continue;

    // prepare track for vertexing
    rtra.sPinvToP();
    rtra.propagate(0);
    rtra.FillCINV();

    // inclusive distributions and PID assignment:
    const int PID  = checkPID(rtra);
    const int chrg = rtra.charge();

    // fill the particle candidates
    if(PID == 0 || PID == PID::pion) {
      if(chrg > 0) {
	Piplus->push_back(rtra);
	//      cout << " pi+ found " << endl;
      }
      if(chrg < 0) {
	Piminus->push_back(rtra);
	//      cout << " pi- found " << endl;
      }
    } // if(PID==0

  } // for(ArteTable<RecoTrack>


  //------------------ Pattern ----------------------
  kshort->combine();
  kshort->prob_collision();

  //------------ Loop over Pattern Matches -----------
  typedef DP_trait<2>::SDPM_l   SDPM_l;
  typedef DP_trait<2>::SDPM_it  SDPM_it;
  typedef DP_trait<2>::SDPM_cit SDPM_cit;

  const SDPM_it PM_end = kshort->matches().end();
  for(SDPM_it PM_it = kshort->matches().begin(); PM_it != PM_end;
      ++PM_it) {
    SDecayPatternMatch<2>& dpm = *PM_it;

    if(!dpm.isValid()) continue;

    float x[1];
    x[0] = dpm.mass();

    Kshort->Fill(x);
  } // for SDPM
  return true;
}

//==============================================================================
// EventCleanup
//==============================================================================
bool K0Analysis::EventCleanup() {
  return true;
}

//==============================================================================
// End processing
//==============================================================================
bool K0Analysis::End() {
  return true;
}

