#!/bin/bash
##########################################################
#
# type:      shell script
#
# created:   05. Dez 2000
#
# author:    Thorsten Glebe
#            HERA-B Collaboration
#            Max-Planck-Institut fuer Kernphysik
#            Saupfercheckweg 1
#            69117 Heidelberg
#            Germany
#            E-mail: T.Glebe@mpi-hd.mpg.de
#
# Description: script to generate ClueRelation 
#              template specializations
#
# 27 Dec 2000 (TG) end() optimization, removed outdated code
# 14 Aug 2001 (TG) added MC_Track-MC_Vertex relation, cleanup
#
##########################################################

CLASSES="MC_Track MC_Vertex TrackClones RecoTrack RecoVertex EcalHit"
RELPAIR="RecoTrack,MC_Track RecoTrack,RecoVertex RecoTrack,TrackClones RecoTrack,EcalHit MC_Track,MC_Vertex MC_Track,MC_Track"
OUTFILE="include/ClueRelationSpecialize.hh"
OUTFILE2="include/ClueRelationDeclaration.hh"
OUTFILE3="include/LinkDef.hh"


if [ -f $OUTFILE ]; then
    echo "rm $OUTFILE"
    rm -f $OUTFILE
fi
if [ -f $OUTFILE2 ]; then
    echo "rm $OUTFILE2"
    rm -f $OUTFILE2
fi
if [ -f $OUTFILE3 ]; then
    echo "rm $OUTFILE3"
    rm -f $OUTFILE3
fi

echo "##########################################################"
echo "# generate $OUTFILE"
echo "# generate $OUTFILE2"
echo "# generate $OUTFILE3"
echo "##########################################################"


##########################################################
# Generate header
##########################################################
(
    echo "#ifndef __ClueRELATIONSPECIALIZE_HH
#define __ClueRELATIONSPECIALIZE_HH
// #########################################################
// This file is automatically generated by
// ClueRelationSpecialize.sh. Do not edit!
// #########################################################

template <class T> class ClueRelationList;

#include <list>
#include \"clue/cluearte.hh\"
"
) > $OUTFILE

(
    echo "#ifndef __ClueRELATIONDECLAR_HH
#define __ClueRELATIONDECLAR_HH
#ifdef  __CINT__
// #########################################################
// This file is automatically generated by
// ClueRelationSpecialize.sh. Do not edit!
// #########################################################
template <class T> class GRelationList;
template <class T> class ClueRelationList;
template <class T> class ClueRelationIterator;
template <class T> class ConstClueRelationIterator;
template <class T> class ReverseClueRelationIterator;
template <class T> class ConstReverseClueRelationIterator;
template <class T> class ArtePointer;
"
) > $OUTFILE2

(
    echo "#ifdef __CINT__
// #########################################################
// This file is automatically generated by
// ClueRelationSpecialize.sh. Do not edit!
// #########################################################

#pragma link C++ class G_trait*;
#pragma link C++ class GRelationList*;
#pragma link C++ class ClueRelationList*;
#pragma link C++ class GRelationIterator*;
#pragma link C++ class ConstGRelationIterator*;
#pragma link C++ class ReverseGRelationIterator*;
#pragma link C++ class ConstReverseGRelationIterator*;
#pragma link C++ class ClueRelationIterator*;
#pragma link C++ class ConstClueRelationIterator*;
#pragma link C++ class ReverseClueRelationIterator*;
#pragma link C++ class ConstReverseClueRelationIterator*;

#pragma link C++ function createRelation*;
#pragma link C++ function createSingleRelation*;
#pragma link C++ function removeRelation*;
#pragma link C++ function removeSingleRelation*;
"
) > $OUTFILE3

##########################################################
# Generate body
##########################################################
for i in $CLASSES; do
(
    echo "
class $i;

//============================================================================
// The $i G-Trait
//============================================================================
template<> struct G_trait<ArtePointer<$i> > {
  // STL independend defs of value_type, pointer, reference
  typedef ArtePointer<$i>                       Gvalue_type;
  typedef ArtePointer<$i>                       Gpointer;
  typedef ArtePointer<$i> const                 Gconst_pointer;
  typedef ArtePointer<$i>&                      Greference;
  typedef const ArtePointer<$i>&                Gconst_reference;
  typedef std::list<Gpointer>                          GRelation_v;
  typedef std::list<Gpointer>::iterator                GRelation_it;
  typedef std::list<Gpointer>::const_iterator          GRelation_cit;
  typedef std::list<Gpointer>::reverse_iterator        GRelation_rit;
  typedef std::list<Gpointer>::const_reverse_iterator  GRelation_rcit;
};


//============================================================================
// GRelationList specializations for $i
//============================================================================
 inline void GRelationList<ArtePointer<$i> >::push_back(const G_trait<ArtePointer<$i> >::Gvalue_type& rhs) { rellist.push_back(rhs.index()); }

inline void GRelationList<ArtePointer<$i> >::unbook(const G_trait<ArtePointer<$i> >::Gvalue_type& rel) { rellist.remove(rel.index()); }

inline void GRelationList<ArtePointer<$i> >::print(std::ostream& os) const {
  typedef G_trait<ArtePointer<$i> >::GRelation_cit GRelation_cit;

  cout << \"Relations: \";
  const GRelation_cit last = --rellist.end();
  const GRelation_cit end  = rellist.end();
  for(GRelation_cit it=rellist.begin(); it!=end; ++it) {
    // **it only valid if value_type is a pointer!!
    // If this is not the case, change to *it!!
    os << *it;
    if(it != last)
      cout << \", \";
  }
  return;
}
"
) >> $OUTFILE

(
    echo "
class $i;
struct G_trait<ArtePointer<$i> >;
std::ostream& operator<<(std::ostream& os, const GRelationList<ArtePointer<$i> >& l);
std::ostream& operator<<(std::ostream& os, const ClueRelationList<$i>& l);
std::ostream& operator<<(std::ostream& os, const ClueRelationIterator<$i>& l);
std::ostream& operator<<(std::ostream& os, const ConstClueRelationIterator<$i>& l);
std::ostream& operator<<(std::ostream& os, const ReverseClueRelationIterator<$i>& l);
std::ostream& operator<<(std::ostream& os, const ConstReverseClueRelationIterator<$i>& l);
"
) >> $OUTFILE2

(
    echo "\
//#pragma link C++ class $i-;
//#pragma link C++ function operator<<( ostream&, const $i& );
//#pragma link C++ class ArtePointer<$i>;
//#pragma link C++ class GRelationList<ArtePointer<$i> >;
#pragma link C++ class ClueRelationList<$i>-!;
#pragma link C++ function operator<<(ostream&, const GRelationList<ArtePointer<$i> >&);
#pragma link C++ function operator<<(ostream&, const ClueRelationList<$i>&);
"
) >> $OUTFILE3
done

# Loop over relation pairs
for i in $RELPAIR; do
  LHStype=`echo $i | cut -d, -f1`
  RHStype=`echo $i | cut -d, -f2`

  for j in 0 1 2 3; do
    if [ $j -eq 0 ]; then ca=""; cb=""; fi
    if [ $j -eq 1 ]; then ca="const"; cb=""; fi
    if [ $j -eq 2 ]; then ca=""; cb="const"; fi
    if [ $j -eq 3 ]; then ca="const"; cb="const"; fi  

    LHS="$ca ArtePointer<$LHStype>&"
    RHS="$cb ArtePointer<$RHStype>&"
    (
      echo "
const bool createSingleRelation($LHS, $RHS);
const bool createSingleRelation($RHS, $LHS);
const bool createRelation($LHS, $RHS);
const bool createRelation($RHS, $LHS);
const bool removeSingleRelation($LHS, $RHS);
const bool removeSingleRelation($RHS, $LHS);
const bool removeRelation($LHS, $RHS);
const bool removeRelation($RHS, $LHS);
"
    ) >> $OUTFILE2
  done

(
echo "void GRelationList<ArtePointer<$LHStype> >::unbook_all(const ArtePointer<$RHStype>&);
void GRelationList<ArtePointer<$RHStype> >::unbook_all(const ArtePointer<$LHStype>&);
"
) >> $OUTFILE2

(
    echo "\
#pragma link C++ function GRelationList<ArtePointer<$LHStype> >::unbook_all(const ArtePointer<$RHStype>&);
#pragma link C++ function GRelationList<ArtePointer<$RHStype> >::unbook_all(const ArtePointer<$LHStype>&);
"
) >> $OUTFILE3
done


##########################################################
# createRelation specialications
##########################################################
for i in 0 1 2 3; do
  if [ $i -eq 0 ]; then ca=""; cb=""; fi
  if [ $i -eq 1 ]; then ca="const"; cb=""; fi
  if [ $i -eq 2 ]; then ca=""; cb="const"; fi
  if [ $i -eq 3 ]; then ca="const"; cb="const"; fi  
    (
    echo "\
//============================================================================
// specializations for createRelation, removeRelation
//============================================================================
template <class L, class R>
const bool createRelation($ca ArtePointer<L>& lhs,
			  $cb ArtePointer<R>& rhs) {

  // container with Rs in L:
  ClueRelationList<R>& rhs_cont = lhs->container(rhs);
  // container with Ls in R:
  ClueRelationList<L>& lhs_cont = rhs->container(lhs);

  rhs_cont.push_back(rhs);
  lhs_cont.push_back(lhs);
  return true;
}


template <class L, class R>
const bool createRelation($ca ArtePointer<L>& lhs, 
			  ClueRelationList<R>& rhs_cont,
			  $cb ArtePointer<R>& rhs) {

  // container with Ls in R:
  ClueRelationList<L>& lhs_cont = rhs->container(lhs);

  rhs_cont.push_back(rhs);
  lhs_cont.push_back(lhs);
  return true;
}

template <class L, class R>
const bool createSingleRelation($ca ArtePointer<L>& lhs,
			        $cb ArtePointer<R>& rhs) {

  // container with Rs in L:
  ClueRelationList<R>& rhs_cont = lhs->container(rhs);

  rhs_cont.push_back(rhs);
  return true;
}

template <class L, class R>
const bool createSingleRelation($ca L& lhs, $cb ArtePointer<R>& rhs) {

  // container with Rs in L:
  ClueRelationList<R>& rhs_cont = lhs.container(rhs);

  rhs_cont.push_back(rhs);
  return true;
}


template <class L, class R>
const bool removeRelation($ca ArtePointer<L>& lhs,
			  $cb ArtePointer<R>& rhs) {

  // container with Rs in L:
  ClueRelationList<R>& rhs_cont = lhs->container(rhs);
  // container with Ls in R:
  ClueRelationList<L>& lhs_cont = rhs->container(lhs);

  rhs_cont.unbook(rhs);
  lhs_cont.unbook(lhs);
  return true;
}

template <class L, class R>
const bool removeRelation($ca ArtePointer<L>& lhs, 
			  ClueRelationList<R>& rhs_cont,
			  $cb ArtePointer<R>& rhs) {

  // container with Ls in R:
  ClueRelationList<L>& lhs_cont = rhs->container(lhs);

  rhs_cont.unbook(rhs);
  lhs_cont.unbook(lhs);
  return true;
}

template <class L, class R>
const bool removeSingleRelation($ca ArtePointer<L>& lhs,
				$cb ArtePointer<R>& rhs) {

  // container with Rs in L:
  ClueRelationList<R>& rhs_cont = lhs->container(rhs);

  rhs_cont.unbook(rhs);
  return true;
}

template <class L, class R>
const bool removeSingleRelation($ca L& lhs, $cb ArtePointer<R>& rhs) {

  // container with Rs in L:
  ClueRelationList<R>& rhs_cont = lhs.container(rhs);

  rhs_cont.unbook(rhs);
  return true;
}
"
    ) >> $OUTFILE
done

##########################################################
# Generate trailer
##########################################################
(
    echo "#endif"
) >> $OUTFILE

(
    echo "#endif // __CINT__
#endif
"
) >> $OUTFILE2

(
    echo "#endif"
) >> $OUTFILE3
