#ifndef __ClueRELATION_HH
#define __ClueRELATION_HH
// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   03. Dez 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: ARTE relation like relation container class
//
// changes:
// 03 Dec 2000 (TG) creation
// 06 Dec 2000 (TG) added grelationfwd.hh
// 10 Dec 2000 (TG) added Streamer method
// 14 Dec 2000 (TG) removed comments from Streamer
// 14 Dec 2000 (TG) bug fix in Streamer: clear container before read
// 27 Dec 2000 (TG) optimization of Streamer()
// 03 Jan 2001 (TG) Streamer() modified
//
// *****************************************************************************

#include <iosfwd>
#include "grelation/grelationfwd.hh"
#include "grelation/GRelationList.hh"
#include "grelation/ClueRelationSpecialize.hh"

template <class T> class ArtePointer;

/** Relation container for the ARTE relation implementation of the BEE
    analysis framework.
    @memo ARTE relation container
    @author T. Glebe
*/
//============================================================================
// ClueRelationList
//============================================================================
template <class T>
class ClueRelationList : public GRelationList<ArtePointer<T> >
{
public:
  /** @name --- Constructors --- */
  ///
  ClueRelationList() : GRelationList<ArtePointer<T> >() {}
  ///
  ClueRelationList(const ClueRelationList<T>& rhs) : 
    GRelationList<ArtePointer<T> >(rhs) {}
  ///
  ~ClueRelationList() {}

#ifdef USE_ROOT
   /** @name --- Expert functions --- */
   /// only available in ROOT
   void Streamer(TBuffer &R__b) {
     if (R__b.IsReading()) {
       rellist.clear();
       Int_t R__i = 0, R__n = 0;
       UShort_t R__t = 0;
       R__b >> R__n;
       //       cout << "x R__n: " << R__n << endl;
       for (R__i = 0; R__i < R__n; ++R__i) {
	 R__b >> R__t;
	 //	 cout << " R__t: " << R__t << endl;
	 const ArtePointer<T> cl(R__t);
	 rellist.push_back(cl);
       }
     } else {
       R__b << static_cast<Int_t>(rellist.size());
       const ClueRelationIterator<T> last = end();
       for (ClueRelationIterator<T> R__k = begin(); R__k != last; ++R__k) {
	 //	 cout << " (*R__k).index(): " << (*R__k).index() << endl;
	 R__b << (*R__k).index();
       }
     }
   }
#endif

}; // ClueRelationList


//==============================================================================
// operator<<
//==============================================================================
template<class T>
inline std::ostream& operator<<(std::ostream& s, const ClueRelationList<T>& a) {
  a.print(s);
  return s;
}


/** ARTE relation like forward iterator.
    @memo Iterator
    @author T. Glebe
*/
//============================================================================
// ClueRelationIterator
//============================================================================
template <class T>
class ClueRelationIterator : public GRelationIterator<ArtePointer<T> > {
public:
  /** @name --- Constructors --- */
  ///
  ClueRelationIterator() {}
  ///
  ClueRelationIterator(const ClueRelationIterator<T>& rhs) : 
    GRelationIterator<ArtePointer<T> >(rhs) {}
  ///
  ClueRelationIterator(const GRelationIterator<ArtePointer<T> >& rhs):
    GRelationIterator<ArtePointer<T> >(rhs) {}
  ///
  ~ClueRelationIterator() {}

}; // ClueRelationIterator


//==============================================================================
// operator<<
//==============================================================================
template<class T>
inline std::ostream& operator<<(std::ostream& s, const ClueRelationIterator<T>& a) {
  s << *a;
  return s;
}


/** ARTE relation like read-only forward iterator.
    @memo Iterator
    @author T. Glebe
*/
//============================================================================
// ConstClueRelationIterator
//============================================================================
template <class T>
class ConstClueRelationIterator : public ConstGRelationIterator<ArtePointer<T> > {
public:
  /** @name --- Constructors --- */
  ///
  ConstClueRelationIterator() {}
  ///
  ConstClueRelationIterator(const ConstClueRelationIterator<T>& rhs) : 
    ConstGRelationIterator<ArtePointer<T> >(rhs) {}
  ///
  ConstClueRelationIterator(const ConstGRelationIterator<ArtePointer<T> >& rhs):
    ConstGRelationIterator<ArtePointer<T> >(rhs) {}
  ///
  ~ConstClueRelationIterator() {}

}; // ConstClueRelationIterator


//==============================================================================
// operator<<
//==============================================================================
template<class T>
inline std::ostream& operator<<(std::ostream& s, const ConstClueRelationIterator<T>& a) {
  s << *a;
  return s;
}


/** ARTE relation like backward iterator.
    @memo Iterator
    @author T. Glebe
*/
//============================================================================
// ReverseClueRelationIterator
//============================================================================
template <class T>
class ReverseClueRelationIterator : public ReverseGRelationIterator<ArtePointer<T> > {
public:
  /** @name --- Constructors --- */
  ///
  ReverseClueRelationIterator() {}
  ///
  ReverseClueRelationIterator(const ReverseClueRelationIterator<T>& rhs) : 
    ReverseGRelationIterator<ArtePointer<T> >(rhs) {}
  ///
  ReverseClueRelationIterator(const ReverseGRelationIterator<ArtePointer<T> >& rhs):
    ReverseGRelationIterator<ArtePointer<T> >(rhs) {}
  ///
  ~ReverseClueRelationIterator() {}

}; // ReverseClueRelationIterator


//==============================================================================
// operator<<
//==============================================================================
template<class T>
inline std::ostream& operator<<(std::ostream& s, const ReverseClueRelationIterator<T>& a) {
  s << *a;
  return s;
}


/** ARTE relation like read-only backward iterator.
    @memo Iterator
    @author T. Glebe
*/
//============================================================================
// ConstReverseClueRelationIterator
//============================================================================
template <class T>
class ConstReverseClueRelationIterator : public ConstReverseGRelationIterator<ArtePointer<T> > {
public:
  /** @name --- Constructors --- */
  ///
  ConstReverseClueRelationIterator() {}
  ///
  ConstReverseClueRelationIterator(const ConstReverseClueRelationIterator<T>& rhs) : 
    ConstReverseGRelationIterator<ArtePointer<T> >(rhs) {}
  ///
  ConstReverseClueRelationIterator(const ConstReverseGRelationIterator<ArtePointer<T> >& rhs):
    ConstReverseGRelationIterator<ArtePointer<T> >(rhs) {}
  ///
  ~ConstReverseClueRelationIterator() {}

}; // ConstReverseClueRelationIterator


//==============================================================================
// operator<<
//==============================================================================
template<class T>
inline std::ostream& operator<<(std::ostream& s, const ConstReverseClueRelationIterator<T>& a) {
  s << *a;
  return s;
}
#endif
