#ifndef __GSRELATION_HH
#define __GSRELATION_HH
// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   03. Dez 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: relation container class
//
// changes:
// 03 Dez 2000 (TG) creation
//
// *****************************************************************************
#include <iosfwd>

//============================================================================
// G simple Pointer class
//============================================================================
class GsPointerBase {
public:
  GsPointerBase() : i(0) {}
  GsPointerBase(unsigned int rhs) : i(rhs) {}
  ~GsPointerBase() {}

  void print(std::ostream& os) const {
    cout << "PTR: [" << i << "]";
  }

  inline bool operator==(const GsPointerBase& rhs) {
    return i == rhs.i;
  }
  inline bool operator!=(const GsPointerBase& rhs) {
    return i != rhs.i;
  }
private:
  unsigned int i;
}; // GsPointerBase


template <class T>
class GsPointer : public GsPointerBase {
public:
  GsPointer() : GsPointerBase(), ptr(0) {}
  GsPointer(T& rhs, unsigned int no) : GsPointerBase(no), ptr(&rhs) {}
  ~GsPointer() {}

  inline T& operator*()  const { return *ptr; }
  inline T* operator->() const { return ptr; }

  inline bool operator==(const GsPointer<T>& rhs) {
    return GsPointerBase::operator==(rhs);
  }
private:
  T* ptr;
}; // GsPointer


//==============================================================================
// operator<<
//==============================================================================
inline std::ostream& operator<<(std::ostream& s, const GsPointerBase& a) {
  a.print(s);
  return s;
}

//============================================================================
// The G-Trait
//============================================================================
#ifndef __CINT__
#ifdef XXX
template<> struct G_trait<GsPointerBase> {
  // STL independend defs of value_type, pointer, reference
  typedef GsPointerBase                                Gvalue_type;
  typedef GsPointerBase                                Gpointer;
  typedef GsPointerBase&                               Greference;
  typedef std::list<Gpointer>                          GRelation_v;
  typedef std::list<Gpointer>::iterator                GRelation_it;
  typedef std::list<Gpointer>::const_iterator          GRelation_cit;
  typedef std::list<Gpointer>::reverse_iterator        GRelation_rit;
  typedef std::list<Gpointer>::const_reverse_iterator  GRelation_rcit;
};
#endif
template<> struct G_trait<GsPointer<Track> > {
  // STL independend defs of value_type, pointer, reference
  typedef GsPointer<Track>                             Gvalue_type;
  typedef GsPointer<Track>                             Gpointer;
  typedef GsPointer<Track> const                       Gconst_pointer;
  typedef GsPointer<Track>&                            Greference;
  typedef std::list<Gpointer>                          GRelation_v;
  typedef std::list<Gpointer>::iterator                GRelation_it;
  typedef std::list<Gpointer>::const_iterator          GRelation_cit;
  typedef std::list<Gpointer>::reverse_iterator        GRelation_rit;
  typedef std::list<Gpointer>::const_reverse_iterator  GRelation_rcit;
};
#else
  typedef GsPointerBase                                Gvalue_type;
  typedef GsPointerBase                                Gpointer;
  typedef GsPointerBase const                          Gconst_pointer;
  typedef GsPointerBase&                               Greference;
  typedef std::list<Gpointer>                          GRelation_v;
  typedef std::list<Gpointer>::iterator                GRelation_it;
  typedef std::list<Gpointer>::const_iterator          GRelation_cit;
  typedef std::list<Gpointer>::reverse_iterator        GRelation_rit;
  typedef std::list<Gpointer>::const_reverse_iterator  GRelation_rcit;
#endif

//============================================================================
// Iterator specializations for GsPointerBase
//============================================================================
class GsPointerBase;

// inline G_trait<GsPointerBase>::Greference GRelationIterator<GsPointerBase>::operator*()  const { return *iter; }

inline G_trait<GsPointer<Track> >::Gpointer
GRelationIterator<GsPointer<Track> >::operator*()  const { return *iter; }

inline void GRelationList<GsPointer<Track> >::push_back(const G_trait<GsPointer<Track> >::Gvalue_type& rhs) { rellist.push_back(rhs); }

inline void GRelationList<GsPointer<Track> >::unbook(const G_trait<GsPointer<Track> >::Gvalue_type& rel) { rellist.remove(const_cast<GsPointer<Track>&>(rel)); }


//============================================================================
// ClueRelationList
//============================================================================
template <class T>
class ClueRelationList : public GRelationList<GsPointer<T> > {
public:
  ClueRelationList() : GRelationList<GsPointer<T> >() {}
}; // ClueRelationIterator


//============================================================================
// ClueRelationIterator
//============================================================================
template <class T>
class ClueRelationIterator : public GRelationIterator<GsPointer<T> > {
public:
  ClueRelationIterator() {}
  ClueRelationIterator(const GRelationIterator<GsPointer<T> >& rhs):
    GRelationIterator<GsPointer<T> >(rhs) {}

}; // ClueRelationIterator


//============================================================================
// specializations for createRelation, removeRelation
//============================================================================
// const overloaded functions missing at the moment!!
template <class L, class R>
const bool createRelation(const GsPointer<L>& lhs,
			  const GsPointer<R>& rhs) {

  // container with R's in L:
  ClueRelationList<R>& rhs_cont = lhs->container(rhs);
  // container with L's in R:
  ClueRelationList<L>& lhs_cont = rhs->container(lhs);

  rhs_cont.push_back(rhs);
  lhs_cont.push_back(lhs);
  return true;
}

template <class L, class R>
const bool createRelation(const GsPointer<L>& lhs, 
			  ClueRelationList<R>& rhs_cont,
			  const GsPointer<R>& rhs) {

  // container with L's in R:
  ClueRelationList<L>& lhs_cont = rhs->container(lhs);

  rhs_cont.push_back(rhs);
  lhs_cont.push_back(lhs);
  return true;
}

template <class L, class R>
const bool createSingleRelation(GsPointer<L>& lhs, GsPointer<R>& rhs) {

  // container with R's in L:
  ClueRelationList<R>& rhs_cont = lhs->container(rhs);

  rhs_cont.push_back(rhs);
  return true;
}

template <class L, class R>
const bool removeRelation(GsPointer<L>& lhs, GsPointer<R>& rhs) {

  // container with R's in L:
  ClueRelationList<R>& rhs_cont = lhs->container(rhs);
  // container with L's in R:
  ClueRelationList<L>& lhs_cont = rhs->container(lhs);

  rhs_cont.unbook(rhs);
  lhs_cont.unbook(lhs);
  return true;
}

template <class L, class R>
const bool removeRelation(GsPointer<L>& lhs, 
			  ClueRelationList<R>& rhs_cont,
			  GsPointer<R>& rhs) {

  // container with L's in R:
  ClueRelationList<L>& lhs_cont = rhs->container(lhs);

  rhs_cont.unbook(rhs);
  lhs_cont.unbook(lhs);
  return true;
}

template <class L, class R>
const bool removeSingleRelation(GsPointer<L>& lhs, GsPointer<R>& rhs) {

  // container with R's in L:
  ClueRelationList<R>& rhs_cont = lhs->container(rhs);

  rhs_cont.unbook(rhs);
  return true;
}

#endif
