#include <algorithm>

#ifndef GROVER_WITHOUT_ARTE
#include "arte/arte.hh"
#include "arte/ArteUtil.h"
#include "marple/marple.hh"
#endif

#include "grover/grover.hh"

// Part of the Grover package
// Please see the header file grover.hh
// for authors etc.
//
//
// Modifications:
// When?     Who?     What?
// ------------------------------------------------------------------------
// 
// 01-07-07  Michael  implementation: select tracks like GroverFillSmarter 
//                    and remove VDSonly tracks afterwards
// 01-07-17  Michael use GroverFillSmarter for consistency

#ifndef GROVER_WITHOUT_ARTE
extern int anl_TrSelect(vector< ArtePointer<RTRA> > &rtra);
#endif

int GroverFillSmartNoVDSonly(GroverParticleList* theList)
{
  ////////////////////////////////////////////////////////////////////////
  //
  // 1) Fills a GroverParticleList according to Michael Schmelling's and 
  //    Patricia Conde's routine for track selection and clone removal
  // 2) No VDSonly tracks are present.
  //
  // Returns SUCCESS if OK, FAILURE if none found.
  //
  ////////////////////////////////////////////////////////////////////////

  GroverParticleList SmartList;
  GroverFillSmarter(&SmartList);

#ifndef GROVER_WITHOUT_ARTE

  int ntr = SmartList.size();

  if (ntr == 0) return FAILURE;
  else {
    for (GroverParticleList::iterator gp = SmartList.begin();
	 gp != SmartList.end(); ++gp) {
      ArtePointer<RTRA> prtra = gp->ParentRTRA();
      if (prtra == 0) 	continue;

      // don't copy VDSonly tracks
      if (prtra->hou <= 0 && prtra->hin <= 0 && prtra->hca <= 0 && 
	  prtra->hri <= 0)  continue;

      theList->push_back(*gp);
    }
  }
#endif


  return (theList->size() > 0) ? SUCCESS : FAILURE;
}
