#ifndef __DECAYPATTERNMATCHIF_HH
#define __DECAYPATTERNMATCHIF_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   21. Aug 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: DecayPatternMatch object interface
//
// changes:
// 21 Aug 2001 (TG) creation
// 28 Aug 2001 (TG) PatternMatch(i) added
// 31 Aug 2001 (TG) moved PatternMatch(i) to PatternMatchIf to allow recursive
//                  call of this member function
//
// ********************************************************************
#include <iosfwd>
#include "interfaces/VertexIf.hh"
#include "interfaces/PatternMatchIf.hh"

class SPatternMatch;
template <class T> class GRelationList;

/** Abstract class which defines the minimal interface for a DecayPatternMatch
    object. PatternMatchIf represents the reconstructed particle, VertexIf
    the vertex.

    @author T. Glebe
*/
//==============================================================================
// DecayPatternMatchIf
//==============================================================================
class DecayPatternMatchIf : public VertexIf, public PatternMatchIf {
public:

  /** @name --- GRelations --- */
  /// relation to SPatternMatch
  virtual GRelationList<SPatternMatch>& container(const SPatternMatch& p) = 0;
  
  /** @name --- Expert methods --- */
  /// used by operator<<()
  virtual std::ostream& print( std::ostream& ) const = 0;
}; // end of class DecayPatternMatchIf


//==============================================================================
// operator <<
//==============================================================================
inline std::ostream& operator<< (std::ostream& os, const DecayPatternMatchIf& t) {
  return t.print(os);
}
#endif
