#!/bin/bash
##########################################################
#
# type:      shell script
#
# created:   05. Dez 2000
#
# author:    Thorsten Glebe
#            HERA-B Collaboration
#            Max-Planck-Institut fuer Kernphysik
#            Saupfercheckweg 1
#            69117 Heidelberg
#            Germany
#            E-mail: T.Glebe@mpi-hd.mpg.de
#
# Description: script to generate GObjArray, GClonesArray
#              template specializations
#
##########################################################

CLASSES=" "
#PatternMatch
OUTFILE1="/dev/null"
PROPCLASSES="RecoTrack,float VERTEX::Vertex,double"
CLMEMBER_INT="flr flt fle flm fit charge hsi hin hou hri hca htr hmu isRICH\
    isOTR isVDS all rls rlt mom"
CLMEMBER_FLOAT="tx ty x y p zref px py pz pt chi2 trprob lre lrmu lrpi\
                lrk lrp lse lsmu lspi lsk lsp lte lth lee lemi leh lmmu lmh lmu lpi lk lp\
                ebrems energy cer xf xfabs isECAL"
VTXMEMBER_int="vndf ntracks"
VTXMEMBER_double="vx vy vz vchi2 vprob vxerr vyerr vzerr vxerr2 vyerr2 vzerr2\
		  dist angle pmaxfrac"
#OUTFILE1="PatternSpecialize.C"
OUTFILE2="include/PatternDeclaration.hh"
OUTFILE3="include/PatternLinkDef.hh"

#if [ -f $OUTFILE1 ]; then
#    echo "rm $OUTFILE1"
#    rm -f $OUTFILE1
#fi
if [ -f $OUTFILE2 ]; then
    echo "rm $OUTFILE2"
    rm -f $OUTFILE2
fi
if [ -f $OUTFILE3 ]; then
    echo "rm $OUTFILE3"
    rm -f $OUTFILE3
fi

echo "##########################################################"
echo "# generate $OUTFILE1"
echo "# generate $OUTFILE2"
echo "# generate $OUTFILE3"
echo "##########################################################"

##########################################################
# Generate header
##########################################################
(
    echo "
// #########################################################
// This file is automatically generated by
// PatternSpecialize.sh. Do not edit!
// #########################################################
#include \"gate/ObjArray.hh\"
#include \"gate/ClonesArray.hh\"
#include \"pattern/PatternMatch.hh\"
#include \"pattern/Pattern.hh\"
#include \"clue/cluearte.hh\"
#include \"clue/RecoTrack.hh\"

// now explicit instantiation of templates
// (needed in case they are not defined elsewhere)
"
) > $OUTFILE1

(
    echo "#ifndef __PATTERNECLAR_HH
#define __PATTERNDECLAR_HH
#ifdef  __CINT__
// #########################################################
// This file is automatically generated by
// PatternSpecialize.sh. Do not edit!
// #########################################################
template <class T> class GObjArray;
template <class T> class GClonesArray;
template <class T> class GObjArrayIterator;
template <class T> class ConstGObjArrayIterator;

template <class C, class T>                     class ClProperty;
template <class C, class T, class X>            class ClExpression;
template <class C, class T>                     class ClMember;
//template <class C, class T, class P>            class ClMemberOnePar;
//template <class C, class T, class P1, class P2> class ClMemberTwoPar;
template <class C, class T>                     class ClConstant;
"
) > $OUTFILE2

(
    echo "//#ifdef __CINT__
// #########################################################
// This file is automatically generated by
// GateSpecialize.sh. Do not edit!
// #########################################################
//#pragma link C++ class GObjArray*;
//#pragma link C++ class GClonesArray*;
//#pragma link C++ class GObjArrayIterator*;
//#pragma link C++ class ConstGObjArrayIterator*;

#pragma link C++ class ClProperty*;
#pragma link C++ class ClExpression*;
#pragma link C++ class ClMember*;
//#pragma link C++ class ClMemberOnePar*;
//#pragma link C++ class ClMemberTwoPar*;
#pragma link C++ class ClConstant*;
"
) > $OUTFILE3
#pragma link C++ class GObjArray*;


##########################################################
# Generate body
##########################################################
for i in $CLASSES; do
(
    echo "\
template class GObjArray<$i*>;
template class GClonesArray<$i*>;
template class GObjArrayIterator<$i*>;
template class ConstGObjArrayIterator<$i*>;
"
) >> $OUTFILE1

(
    echo "\
class $i;
std::ostream& operator<<(std::ostream& os, const GObjArray<$i*>& l);
std::ostream& operator<<(std::ostream& os, const GClonesArray<$i*>& l);
std::ostream& operator<<(std::ostream& os, const GObjArrayIterator<$i*>& l);
std::ostream& operator<<(std::ostream& os, const ConstGObjArrayIterator<$i*>& l);
"
) >> $OUTFILE2

(
    echo "\
//#pragma link C++ class GObjArray<$i*>;
//#pragma link C++ class GClonesArray<$i*>;
//#pragma link C++ function operator<<(ostream&, const GObjArray<$i*>&);
//#pragma link C++ function operator<<(ostream&, const GObjArrayIterator<$i*>&);
//#pragma link C++ function operator<<(ostream&, const ConstGObjArrayIterator<$i*>&);
//#pragma link C++ function operator<<(ostream&, const GClonesArray<$i*>&);
"
) >> $OUTFILE3
done


for x in $PROPCLASSES; do
    i=`echo $x | cut -d, -f1`
    j=`echo $x | cut -d, -f2`
(
    
    echo "\
class $i;
std::ostream& operator<<(std::ostream& os, const ClProperty<$i,int>& l);
std::ostream& operator<<(std::ostream& os, const ClProperty<$i,$j>& l);
std::ostream& operator<<(std::ostream& os, const ClExpression<$i,int,int>& l);
std::ostream& operator<<(std::ostream& os, const ClExpression<$i,$j,int>& l);
//std::ostream& operator<<(std::ostream& os, const ClExpression<$i,$j,$j>& l);
std::ostream& operator<<(std::ostream& os, const ClMember<$i,int>& l);
std::ostream& operator<<(std::ostream& os, const ClMember<$i,$j>& l);
//std::ostream& operator<<(std::ostream& os, const ClMemberOnePar<$i,int,int>& l);
//std::ostream& operator<<(std::ostream& os, const ClMemberOnePar<$i,$j,int>& l);
//std::ostream& operator<<(std::ostream& os, const ClMemberOnePar<$i,$j,$j>& l);
//std::ostream& operator<<(std::ostream& os, const ClMemberTwoPar<$i,$j,int,int>& l);
std::ostream& operator<<(std::ostream& os, const ClConstant<$i,int>& l);
std::ostream& operator<<(std::ostream& os, const ClConstant<$i,$j>& l);
"
) >> $OUTFILE2

(
    echo "\
#pragma link C++ class    ClProperty<$i,int>-;
#pragma link C++ class    ClProperty<$i,$j>-;
#pragma link C++ class    ClExpression<$i,int,int>;
#pragma link C++ class    ClExpression<$i,$j,int>;
//#pragma link C++ class    ClExpression<$i,$j,$j>;
#pragma link C++ class    ClMember<$i,int>;
#pragma link C++ class    ClMember<$i,$j>;
//#pragma link C++ class    ClMemberOnePar<$i,int,int>;
//#pragma link C++ class    ClMemberOnePar<$i,$j,$j>;
//#pragma link C++ class    ClMemberTwoPar<$i,$j,int,int>;
#pragma link C++ class    ClConstant<$i,int>;
#pragma link C++ class    ClConstant<$i,$j>;

#pragma link C++ function operator<<(ostream&, const ClProperty<$i,int>&);
#pragma link C++ function operator<<(ostream&, const ClProperty<$i,$j>&);
#pragma link C++ function operator<<(ostream&, const ClExpression<$i,int,int>&);
#pragma link C++ function operator<<(ostream&, const ClExpression<$i,$j,int>&);
//#pragma link C++ function operator<<(ostream&, const ClExpression<$i,$j,$j>&);
#pragma link C++ function operator<<(ostream&, const ClMember<$i,int>&);
#pragma link C++ function operator<<(ostream&, const ClMember<$i,$j>&);
//#pragma link C++ function operator<<(ostream&, const ClMemberOnePar<$i,int,int>&);
//#pragma link C++ function operator<<(ostream&, const ClMemberOnePar<$i,$j,$j>&);
//#pragma link C++ function operator<<(ostream&, const ClMemberTwoPar<$i,$j,int,int>&);
#pragma link C++ function operator<<(ostream&, const ClConstant<$i,int>&);
#pragma link C++ function operator<<(ostream&, const ClConstant<$i,$j>&);
"
) >> $OUTFILE3
done

##########################################################
# Appendix
##########################################################
# ClMembers must come after #pragma definitions of ClProperty etc

for i in $CLMEMBER_INT; do
(
    echo "extern ClMember<RecoTrack,int> $i;"
) >> $OUTFILE2
done
for i in $VTXMEMBER_int; do
(
    echo "extern ClMember<VERTEX::Vertex,int> $i;"
) >> $OUTFILE2
done

for i in $CLMEMBER_INT $VTXMEMBER_int; do
(
    echo "#pragma link C++ global $i;"
) >> $OUTFILE3
done
for i in $CLMEMBER_FLOAT; do
(
    echo "extern ClMember<RecoTrack,float> $i;"
) >> $OUTFILE2
done
for i in $VTXMEMBER_double; do
(
    echo "extern ClMember<VERTEX::Vertex,double> $i;"
) >> $OUTFILE2
done

for i in $CLMEMBER_FLOAT $VTXMEMBER_double; do
(
    echo "#pragma link C++ global $i;"
) >> $OUTFILE3
done

##########################################################
# Appendix outside #ifdef __CINT__
##########################################################
echo "\
#endif // CINT
#ifndef __CINT__
" >> $OUTFILE2

for x in $PROPCLASSES; do
    i=`echo $x | cut -d, -f1`
    j=`echo $x | cut -d, -f2`
(
    echo "\
TBuffer& operator>>( TBuffer& buf, int   ($i::* & )() const ) { return buf; }
TBuffer& operator>>( TBuffer& buf, $j    ($i::* & )() const ) { return buf; }
TBuffer& operator<<( TBuffer& buf, int   ($i::* & )() const ) { return buf; }
TBuffer& operator<<( TBuffer& buf, $j    ($i::* & )() const ) { return buf; }

TBuffer& operator>>( TBuffer& buf, int   ($i::* & )(int) const ) { return buf; }
TBuffer& operator>>( TBuffer& buf, $j    ($i::* & )(int) const ) { return buf; }
TBuffer& operator<<( TBuffer& buf, int   ($i::* & )(int) const ) { return buf; }
TBuffer& operator<<( TBuffer& buf, $j    ($i::* & )(int) const ) { return buf; }

TBuffer& operator>>( TBuffer& buf, int   ($i::* & )($j) const ) { return buf; }
TBuffer& operator>>( TBuffer& buf, $j    ($i::* & )($j) const ) { return buf; }
TBuffer& operator<<( TBuffer& buf, int   ($i::* & )($j) const ) { return buf; }
TBuffer& operator<<( TBuffer& buf, $j    ($i::* & )($j) const ) { return buf; }

TBuffer& operator>>( TBuffer& buf, int   ($i::* & )(int,int) const ) { return buf; }
TBuffer& operator>>( TBuffer& buf, $j    ($i::* & )(int,int) const ) { return buf; }
TBuffer& operator<<( TBuffer& buf, int   ($i::* & )(int,int) const ) { return buf; }
TBuffer& operator<<( TBuffer& buf, $j    ($i::* & )(int,int) const ) { return buf; }
"
) >> $OUTFILE2
done

##########################################################
# Generate trailer
##########################################################
(
    echo "\
#endif // __CINT__
#endif
"
) >> $OUTFILE2

(
    echo "//#endif"
) >> $OUTFILE3
