// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   28. Jun 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: A PatternMatch container class
//
// changes:
// 28 Jun 2001 (TG) creation
// 28 Jun 2001 (TG) removed template parameter C and T
// 22 Aug 2001 (TG) added with()
// 28 Aug 2001 (TG) changed push_back(TrackIf) to push_back(MC_Track)
//
// ********************************************************************
#include "pattern/SPatternMatchContainer.hh"

//==============================================================================
// Constructors
//==============================================================================
SPatternMatchContainer::SPatternMatchContainer() : 
  pmlist(),
  ptrlist(),
  pattern_(SmartPointer<SPattern>())
{}

SPatternMatchContainer::SPatternMatchContainer(const SPattern& rhs) : 
  pmlist(),
  ptrlist(),
  pattern_(SmartPointer<SPattern>(rhs))
{ pattern_->set_container(*this); }

SPatternMatchContainer::SPatternMatchContainer(const SmartPointer<SPattern>& rhs) : 
  pmlist(),
  ptrlist(),
  pattern_(rhs)
{ pattern_->set_container(*this); }

SPatternMatchContainer::~SPatternMatchContainer() {}

//==============================================================================
// push_back
//==============================================================================
SPatternMatch& SPatternMatchContainer::push_back(RecoSegment& rs)  { 

  pmlist.push_back(SPatternMatch( rs, pattern_ )); 
  SPatternMatch& pm = pmlist.back();
  ptrlist.push_back(&pm); 
  return pm;
}

SPatternMatch& SPatternMatchContainer::push_back(MC_Track& mtra) { 

  pmlist.push_back(SPatternMatch( mtra, pattern_ ));
  SPatternMatch& pm = pmlist.back();
  ptrlist.push_back(&pm); 
  return pm;
}

//==============================================================================
// with
//==============================================================================
void SPatternMatchContainer::with(const SProperty<TrackIf,bool>& rhs)
{ pattern_->setProperty(rhs); }

//==============================================================================
// select
//==============================================================================
template <class Input>
bool SPatternMatchContainer::select(const Input& src) {
  typedef typename Input::const_iterator Cit;

  pmlist.clear();
  ptrlist.clear();

  const Cit end = src.end();
  for(Cit it = src.begin(); it != end; ++it) {
    const typename Input::value_type& track = *it;

    if( pattern_->evalTrack(*track) ) {
      SPatternMatch pm( *track, pattern_ );
      pmlist.push_back(pm);
      ptrlist.push_back(&pm);
    }
  }
  return true;
}

//==============================================================================
// print
//==============================================================================
std::ostream& SPatternMatchContainer::print(std::ostream& os) const {

  copy(pmlist.begin(), pmlist.end(), ostream_iterator<PatternMatchIf>(os, "\n"));
  return os;
}
