#ifndef P_TRAIT_HH
#define P_TRAIT_HH
// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   25. Jun 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: trait to configure SPattern classes
//
// changes:
// 25 Jun 2001 (TG) creation
// 27 Jun 2001 (TG) added #include GCombiner.hh
// 28 Jun 2001 (TG) remove P_trait
// 02 Jul 2001 (TG) added typedefs for [Decay]PatternMatch pointer
//
// *****************************************************************************

#include <list>

#ifndef __CINT__
#include "gcombiner/GSCombiner.hh"
#endif

class TrackIf;
class SPattern;
class SPatternMatch;
class PatternMatchIf;
class SPatternMatchContainer;
template <class C, class T> class SProperty;
template <class T> class SmartPointer;
template <unsigned int NTR> class SDecayPattern;
template <unsigned int NTR> class SDecayPatternMatch;
template <unsigned int NTR> class SDecayPatternMatchContainer;
template <unsigned int NTR> class SPatternExpression;


//============================================================================
// Some global typedefs
//============================================================================
///
typedef list<SPatternMatch>                  SPM_l;
///
typedef SPM_l::iterator                      SPM_it;
///
typedef SPM_l::const_iterator                SPM_cit;

///
typedef list<PatternMatchIf*>                SPMP_l;
///
typedef SPMP_l::iterator                     SPMP_it;
///
typedef SPMP_l::const_iterator               SPMP_cit;

#ifndef __CINT__
///
typedef GSCombiner<SPMP_l>                   GSC;
//typedef GSCombiner<list<TrackIf*> >                   GSC;
///
typedef GSC::iterator                        GSC_it;
#endif

#ifdef XXX
///
typedef GCombiner<SPM_l>                    PMC;
///
typedef GCombiner<SPM_l>::iterator          PMC_it;
#endif

///
typedef SmartPointer<SPattern>               SP_sp;
///
typedef SmartPointer<SPatternMatchContainer> PMC_sp;


 
/** Configuration of types used in the DecayPatternMatch classes

    @memo type configuration
    @author T. Glebe
 */
//============================================================================
// The DP-Trait (for SDecayPatternMatches)
//============================================================================
template <unsigned int NTR>
struct DP_trait {

  ///
  typedef list<SDecayPatternMatch<NTR> >      SDPM_l;
  ///
  typedef SDPM_l::iterator                    SDPM_it;
  ///
  typedef SDPM_l::const_iterator              SDPM_cit;

  ///
  typedef list<SDecayPatternMatch<NTR>*>      SDPMP_l;
  ///
  typedef SDPMP_l::iterator                   SDPMP_it;
  ///
  typedef SDPMP_l::const_iterator             SDPMP_cit;

  ///
  typedef SmartPointer<SDecayPattern<NTR> >   SDP_sp;

  ///
  typedef SmartPointer<SDecayPatternMatchContainer<NTR> > DPMC_sp;
}; // end of DP_trait
#endif
