#ifndef SPATTERNEXPRESSION_HH
#define SPATTERNEXPRESSION_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   26. Jun 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: [Decay]PatternMatchContainer expression class
//
// changes:
// 26 Jun 2001 (TG) creation
// 28 Jun 2001 (TG) removed template parameter C and T
// 02 Jul 2001 (TG) changed type of pmlist to SPMP_l
// 04 Jul 2001 (TG) operator& added, CTOR added
// 17 Jul 2001 (TG) operator& added, CTOR added
//
// ********************************************************************
#include <iosfwd>
#include "pattern/P_trait.hh"
#include "pattern/SPatternMatchContainer.hh"

class ToppIf;
template <unsigned int NTR> class SDecayPattern;

/** Class which represents a [Decay]PatternMatchContainer expression.

    @memo Generic class
    @author T. Glebe
*/
//==============================================================================
// SPatternExpression
//==============================================================================
template <unsigned int NTR>
class SPatternExpression {

  typedef typename DP_trait<NTR>::SDP_sp         SDP_sp;

public:
  /** @name --- Constructors --- */
  ///
  SPatternExpression();
  ///
  SPatternExpression(const SDecayPattern<NTR>& dp,
		     const PMC_sp& lhs, const PMC_sp& rhs);
  ///
  template <unsigned int NTR2>
  SPatternExpression(const SDecayPattern<NTR>& dp,
		     const PMC_sp& lhs, const
		     SmartPointer<SDecayPatternMatchContainer<NTR2> >& rhs);
  ///
  SPatternExpression(const SDP_sp& dp,
		     const PMC_sp& lhs, const PMC_sp& rhs);
  ///
  SPatternExpression(const ToppIf& topp,
		     const SPatternExpression<NTR>& rhs);
  ///
  SPatternExpression(const SDecayPattern<NTR>& dp,
		     const SPatternExpression<NTR-1>& lhs,
		     const PMC_sp& rhs);
  ///
  template <unsigned int NTR2>
  SPatternExpression(const SDecayPattern<NTR>& dp,
		     const SPatternExpression<NTR-1>& lhs, const
		     SmartPointer<SDecayPatternMatchContainer<NTR2> >& rhs);

  ///
  template <unsigned int NTR1, unsigned int NTR2>
  SPatternExpression(const SDecayPattern<NTR>& dp,
                     const SmartPointer<SDecayPatternMatchContainer<NTR1> >& lhs,
                     const SmartPointer<SDecayPatternMatchContainer<NTR2> >& rhs);
  
  /** @name --- Access methods --- */
  /// get decay pattern
  const SDecayPattern<NTR>& dpattern() const;
#ifdef XXX
  ///
  PMC_sp pcList(unsigned int i) const;
#endif
  ///
  const SPMP_l* pmList(unsigned int i) const;

private:
  const SPMP_l* pmlist[NTR]; // input: pattern match lists
  SDP_sp        dpattern_;   // the decay pattern
}; // end of SPatternExpression


//==============================================================================
// operator&
//==============================================================================
inline SPatternExpression<2> operator&(const PMC_sp& lhs, const PMC_sp& rhs) {

  typedef SPatternExpression<2> SPE2;
  typedef SDecayPattern<2>      SDP2;
  return SPE2(SDP2(Any, lhs->pattern() & rhs->pattern()), lhs, rhs);
}

//==============================================================================
// operator&
//==============================================================================
template <unsigned int NTR>
inline SPatternExpression<NTR> operator&(
      const SPatternExpression<NTR-1>& lhs,
      const PMC_sp& rhs) {

  typedef SPatternExpression<NTR> SPE;
  return SPE(lhs.dpattern() & rhs->pattern(), lhs, rhs);
}

//==============================================================================
// operator&
//==============================================================================
template <unsigned int NTR2>
inline SPatternExpression<2> operator&(
      const PMC_sp& lhs,
      const SmartPointer<SDecayPatternMatchContainer<NTR2> >& rhs) {

  typedef SPatternExpression<2> SPE2;
  typedef SDecayPattern<2>      SDP2;
  return SPE2(SDP2(Any, lhs->pattern() & rhs->pattern()), lhs, rhs);
}

//==============================================================================
// operator&
//==============================================================================
template <unsigned int NTR2>
inline SPatternExpression<2> operator&(
      const SmartPointer<SDecayPatternMatchContainer<NTR2> >& lhs,
      const PMC_sp& rhs) {

  typedef SPatternExpression<2> SPE2;
  typedef SDecayPattern<2>      SDP2;
  return SPE2(SDP2(Any, lhs->pattern() & rhs->pattern()), rhs, lhs);
}

//==============================================================================
// operator&
//==============================================================================
template <unsigned int NTR, unsigned int NTR2>
inline SPatternExpression<NTR> operator&(
      const SPatternExpression<NTR-1>& lhs,					 
      const SmartPointer<SDecayPatternMatchContainer<NTR2> >& rhs) {

  typedef SPatternExpression<NTR> SPE;
  typedef SDecayPattern<NTR>      SDP;
  return SPE(SDP(Any, lhs.dpattern() & rhs->pattern()), lhs, rhs);
}

//==============================================================================
// operator&
//==============================================================================
template <unsigned int NTR>
inline SPatternExpression<2> operator&(
        const SmartPointer<SDecayPatternMatchContainer<NTR> >& lhs,
        const SmartPointer<SDecayPatternMatchContainer<NTR> >& rhs) {

  typedef SPatternExpression<2> SPE;
  typedef SDecayPattern<2>      SDP;
  return SPE(SDP(Any, lhs->pattern() & rhs->pattern()), lhs, rhs);
}

//==============================================================================
// operator&
//==============================================================================
template <unsigned int NTR1, unsigned int NTR2>
inline SPatternExpression<2> operator&(
      const SmartPointer<SDecayPatternMatchContainer<NTR1> >& lhs,
      const SmartPointer<SDecayPatternMatchContainer<NTR2> >& rhs) {

  typedef SPatternExpression<2> SPE;
  typedef SDecayPattern<2>      SDP;
  return SPE(SDP(Any, lhs->pattern() & rhs->pattern()), lhs, rhs);
}

#include "pattern/SPatternExpression.icc"
#endif
