// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   01. Feb 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: ROOT specific class describing a decay pattern
//              for RecoTracks
//
// changes:
// 01 Feb 2001 (TG) creation
//
// ********************************************************************
#include <iostream>
#include "clue/cluearte.hh"  // needed by RecoTrack
#include "clue/RecoTrack.hh" // needed by SmartPointer->print

#include "pattern/DecayPatternMatch.hh"
#include "pattern/DecayPattern.hh"
#include "pattern/Pattern.hh"

//==============================================================================
// Constructors
//==============================================================================
DecayPattern::DecayPattern() : DPatternBase()
{}

DecayPattern::DecayPattern(const DPatternBase& rhs) :
  DPatternBase(rhs)
{}

DecayPattern::DecayPattern( const ToppIf& tp, const DPatternBase& rhs) :
  DPatternBase(tp, rhs)
{}

DecayPattern::DecayPattern( const ToppIf& tp, const DecayPattern& rhs) :
  DPatternBase(tp, rhs)
{}

DecayPattern::DecayPattern( const DecayPattern& rhs) :
  DPatternBase(rhs)
{}

DecayPattern::DecayPattern( const ToppIf& tp, const Pattern& lhs, const Pattern& rhs) :
  DPatternBase(tp, lhs, rhs)
{}

DecayPattern::~DecayPattern()
{}

//==============================================================================
// operator&
//==============================================================================
DecayPattern& DecayPattern::operator&( const Pattern& rhs) {
  DPatternBase::operator&(rhs);
  return *this;
}

DecayPattern& DecayPattern::operator&( const DecayPattern& rhs) {
  DPatternBase::operator&(rhs);
  return *this;
}


//==============================================================================
// select
//==============================================================================
/// forwarding function
bool DecayPattern::select(const RT_t& src, DPM_l& trg) {
  return DPatternBase::select(src,trg, DecayPatternMatch());
}



//==============================================================================
// with
//==============================================================================
/// dummy function for right ROOT instantiation order
inline void DecayPattern::with(const VtxProp2& rhs) const {
  cerr << " This function is a ROOT hack!" << endl;
}



#ifdef USE_ROOT
ClassImp(DecayPattern)
#endif
