#ifndef __DECAYPATTERNMATCHIF_HH
#define __DECAYPATTERNMATCHIF_HH
// ********************************************************************
//
// source:
//
// type:      source code
//
// created:   06. Jun 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: DecayPatternMatch object interface
//
// changes:
// 06 Jun 2001 (TG) creation
// 11 Jun 2001 (TG) added mass(),massError()
// 03 Jul 2001 (TG) added SPatternMatch inheritance, added CTOR
// 06 Jul 2001 (TG) added scale argument to massProb()
// 09 Jul 2001 (TG) moved pmatch from SDecayPatternMatch to this place
//
// ********************************************************************
#include <iosfwd>
#include "interfaces/VertexIf.hh"
#include "pattern/SPatternMatch.hh"

class TrackIf;
class SPattern;

/** Abstract class which defines the minimal interface for a DecayPatternMatch object.

    @author T. Glebe
*/
//==============================================================================
// DecayPatternMatchIf
//==============================================================================
class DecayPatternMatchIf : public VertexIf, public SPatternMatch {
public:
  /** @name --- Constructors --- */
  ///
  DecayPatternMatchIf(TrackIf& t) : 
    SPatternMatch(t),
    pmatch()
    {}
  ///
  DecayPatternMatchIf(TrackIf& t, const SPattern& pat) : 
    SPatternMatch(t, pat),
    pmatch()
    {}
  ///
  virtual ~DecayPatternMatchIf() {
    pmatch.unbook_all(*this);
  }

  /** @name --- Access methods --- */
  /// invariant mass of pattern
  virtual double mass() const = 0;
  /// error of inv. mass
  virtual double massError() const = 0;
  /// $\chi^2$ probability including mass-term
  virtual double massProb(double scale) const = 0;

  /** @name --- Expert methods --- */
  /// used by operator<<()
  virtual std::ostream& print( std::ostream& ) const = 0;

  /** @name --- Relations --- */
  /// DecayPatternMatch $\rightarrow$ PatternMatch (relation to outgoing tracks)
  GRelationList<SPatternMatch> pmatch; //! not persistent
  ///
  const GRelationList<SPatternMatch>& Pmatch() const;
  ///
  GRelationList<SPatternMatch>& Pmatch();
  ///
  GRelationList<SPatternMatch>& container(const SPatternMatch& p);
}; // end of class TrackIf


//==============================================================================
// operator <<
//==============================================================================
std::ostream& operator<< (std::ostream& os, const DecayPatternMatchIf& t);

#include "pattern/DecayPatternMatchIf.icc"
#endif
